@@ -17,12 +17,11 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include <boost/signals2/signal.hpp>
 #include <pcrecpp.h>
 
 #include "wb_sql_editor_form.h"
+#include "wb_sql_editor_panel.h"
 #include "query_side_palette.h"
 
 #include "wb_sql_editor_snippets.h"
@@ -35,6 +34,7 @@
 #include "base/geometry.h"
 #include "base/notifications.h"
 #include "base/drawing.h"
+#include "base/log.h"
 
 #include "mforms/toolbar.h"
 #include "mforms/hypertext.h"
@@ -44,21 +44,24 @@
 #include "mforms/code_editor.h"
 
 
-#define USER_SNIPPETS "My Snippets"
+DEFAULT_LOG_DOMAIN("QuerySidebar");
 
 using namespace mforms;
 using namespace base;
 
-//----------------- SnippetList --------------------------------------------------------------------
+//----------------- SnippetListView --------------------------------------------------------------------
 
 /**
   * Internal class containing a list of snippet entries in a scrollable list.
   */
-class SnippetList : public BaseSnippetList
+class SnippetListView : public BaseSnippetList
 {
 private:
+  friend class QuerySidePalette;
+
   wb::SnippetPopover* _snippet_popover;
   bool _user_snippets_active;
+  bool _shared_snippets_active;
 
 private:
 
@@ -85,8 +88,8 @@ private:
   void prepare_context_menu()
   {
     _context_menu = manage(new Menu());
-    _context_menu->set_handler(boost::bind(&SnippetList::on_action, this, _1));
-    _context_menu->signal_will_show()->connect(boost::bind(&SnippetList::menu_will_show, this));
+    _context_menu->set_handler(boost::bind(&SnippetListView::on_action, this, _1));
+    _context_menu->signal_will_show()->connect(boost::bind(&SnippetListView::menu_will_show, this));
 
     _context_menu->add_item(_("Insert Snippet at Cursor"), "insert_text");
     _context_menu->add_item(_("Replace Editor Content with Snippet"), "replace_text");
@@ -105,13 +108,16 @@ private:
 
   void menu_will_show()
   {
+    bool shared_usable = model()->shared_snippets_usable();
+
     _context_menu->set_item_enabled(0, _selected_index > -1);
     _context_menu->set_item_enabled(1, _selected_index > -1);
     _context_menu->set_item_enabled(2, _selected_index > -1);
     _context_menu->set_item_enabled(4, _selected_index > -1);
-    _context_menu->set_item_enabled(6, _selected_index > -1);
-    _context_menu->set_item_enabled(8, _selected_index > -1);
-    _context_menu->set_item_enabled(10, !_user_snippets_active);
+    _context_menu->set_item_enabled(6, _selected_index > -1 && (!_shared_snippets_active || shared_usable));
+    _context_menu->set_item_enabled(7, (!_shared_snippets_active || shared_usable));
+    _context_menu->set_item_enabled(8, _selected_index > -1 && (!_shared_snippets_active || shared_usable));
+    _context_menu->set_item_enabled(10, !_user_snippets_active && !_shared_snippets_active);
   }
 
   //------------------------------------------------------------------------------------------------
@@ -134,30 +140,47 @@ private:
   //------------------------------------------------------------------------------------------------
 
 public:
-  SnippetList(const std::string& icon_name)
+  SnippetListView(const std::string& icon_name)
   : BaseSnippetList(icon_name, DbSqlEditorSnippets::get_instance())
   {
     _user_snippets_active = false;
+    _shared_snippets_active = false;
 
     _snippet_popover = new wb::SnippetPopover();
     _snippet_popover->set_size(376, 257);
-    _snippet_popover->signal_closed()->connect(boost::bind(&SnippetList::popover_closed, this));
+    _snippet_popover->signal_closed()->connect(boost::bind(&SnippetListView::popover_closed, this));
 
     prepare_context_menu();
   }
 
   //------------------------------------------------------------------------------------------------
 
-  ~SnippetList()
+  ~SnippetListView()
   {
     delete _snippet_popover;
     _context_menu->release();
   }
 
+
+  //------------------------------------------------------------------------------------------------
+
+  void edit_new_snippet()
+  {
+    if (!_snippets.empty())
+    {
+      _selected_index = (int)_snippets.size() - 1;
+      _selected_snippet = _snippets.back();
+      edit_snippet(_selected_snippet);
+      _snippet_popover->set_read_only(false);
+    }
+  }
+
   //------------------------------------------------------------------------------------------------
 
   std::string selected_category() { return model()->selected_category(); }
 
+  bool shared_snippets_active() const { return _shared_snippets_active; }
+
   //------------------------------------------------------------------------------------------------
 
   /**
@@ -166,7 +189,15 @@ public:
   void show_category(std::string category)
   {
     _user_snippets_active = (category == USER_SNIPPETS);
-    model()->select_category(category);
+    _shared_snippets_active = (category == SHARED_SNIPPETS);
+    try
+    {
+      model()->select_category(category);
+    }
+    catch (std::exception &exc)
+    {
+      log_warning("Error switching snippet category: %s\n", exc.what());
+    }
     refresh_snippets();
   }
 
@@ -220,7 +251,7 @@ public:
 
   //------------------------------------------------------------------------------------------------
 
-  // End of internal class SnippetList.
+  // End of internal class SnippetListView.
 };
 
 //----------------- QuerySidePalette ---------------------------------------------------------------
@@ -241,6 +272,8 @@ QuerySidePalette::QuerySidePalette(const SqlEditorForm::Ref &owner)
   _switching_help = false;
   _help_task = GrtThreadedTask::create(_grtm);
 
+  _pending_snippets_refresh = true;
+
   mforms::Box *help_page = manage(new Box(false));
   _help_toolbar = prepare_help_toolbar();
   _help_text = manage(new mforms::HyperText());
@@ -270,7 +303,7 @@ QuerySidePalette::QuerySidePalette(const SqlEditorForm::Ref &owner)
   Box* snippet_page = manage(new Box(false));
 
   content_border = manage(new Box(false));
-  _snippet_list = manage(new SnippetList("snippet_sql.png"));
+  _snippet_list = manage(new SnippetListView("snippet_sql.png"));
 #ifdef _WIN32
   content_border->set_padding(3, 3, 3, 3);
   _snippet_list->set_back_color(base::Color::get_application_color_as_string(AppColorPanelContentArea, false));
@@ -317,7 +350,7 @@ QuerySidePalette::~QuerySidePalette()
   base::NotificationCenter::get()->remove_observer(this);
 
   cancel_timer();
-  if (_help_task->is_busy() && _help_task->task() != NULL)
+  if (_help_task->is_busy() && _help_task->task())
     _help_task->task()->cancel();
 }
 
@@ -331,6 +364,19 @@ void QuerySidePalette::cancel_timer()
 
 //--------------------------------------------------------------------------------------------------
 
+static bool contains_editor(SqlEditorForm::Ref form, MySQLEditor *ed)
+{
+  for (int c = form->sql_editor_count(), i = 0; i < c; i++)
+  {
+    SqlEditorPanel *panel = form->sql_editor_panel(i);
+    if (panel && panel->editor_be().get() == ed)
+      return true;
+  }
+  return false;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void QuerySidePalette::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
 {
   // Selection and caret changes notification.
@@ -342,17 +388,20 @@ void QuerySidePalette::handle_notification(const std::string &name, void *sender
     if (code_editor == NULL)
       return;
 
-    Sql_editor *editor = static_cast<Sql_editor*>(code_editor->get_host());
+    MySQLEditor *editor = static_cast<MySQLEditor*>(code_editor->get_host());
     if (editor != NULL && editor->grtobj().is_valid())
     {
-      check_format_structures(editor);
-      cancel_timer();
-      _help_timer = _grtm->run_every(boost::bind(&QuerySidePalette::find_context_help, this, editor), 0.7);
+      // See if this editor instance is actually from the IDE this palette sits in.
+      SqlEditorForm::Ref form = _owner.lock();
+      if (form && contains_editor(form, editor))
+      {
+        check_format_structures(editor);
+        cancel_timer();
+        _help_timer = _grtm->run_every(boost::bind(&QuerySidePalette::find_context_help, this, editor), 0.7);
+      }
     }
-
-    return;
   }
-} 
+}
 
 //--------------------------------------------------------------------------------------------------
 
@@ -378,9 +427,9 @@ void QuerySidePalette::show_help_text_for_topic(const std::string &topic)
     return;
   }
 
-  // Only run one query at a time. There's no sense in issuing new queries if one is still pending
-  // which might indicate a slow server or broken connection.
-  if (_help_task->is_busy() || (_help_task->task() != NULL && _help_task->task()->is_cancelled()))
+  // Add a new task to retrieve the actual help text in the background, but only
+  // if the help task wasn't canceled, which indicates we are probably going down right now.
+  if (_help_task->task() && _help_task->task()->is_cancelled())
     return;
 
   _last_topic = topic_upper;
@@ -419,8 +468,8 @@ grt::StringRef QuerySidePalette::get_help_text_threaded(grt::GRT *)
     _last_topic = "";
   }
 
-  if (!_help_task->task()->is_cancelled())
-    _help_task->execute_in_main_thread(boost::bind(&QuerySidePalette::update_help_ui, this), false, false);
+  // At this point the previous task has already finished (it's the one that triggered this function).
+  _help_task->execute_in_main_thread(boost::bind(&QuerySidePalette::update_help_ui, this), false, false);
 
   return "";
 }
@@ -474,7 +523,7 @@ void QuerySidePalette::show_help_hint_or_update()
 /**
  * Triggered by timer or manually to find a help topic from the given editor's text + position.
  */
-bool QuerySidePalette::find_context_help(Sql_editor *editor)
+bool QuerySidePalette::find_context_help(MySQLEditor *editor)
 {
   _help_timer = NULL;
 
@@ -485,15 +534,15 @@ bool QuerySidePalette::find_context_help(Sql_editor *editor)
   if (editor == NULL)
   {
     SqlEditorForm::Ref form = _owner.lock();
-    Sql_editor::Ref editor_ref = form->active_sql_editor();
-    if (editor_ref)
-      editor = &(*editor_ref);
+    SqlEditorPanel *panel = form->active_sql_editor_panel();
+    if (panel)
+      editor = panel->editor_be().get();
     else
       return false;
   }
 
   // Caret position as <column, row>.
-  std::pair<int, int> caret = editor->cursor_pos_row_column(true);
+  std::pair<size_t, size_t> caret = editor->cursor_pos_row_column(true);
 
   _help_task->exec(false, boost::bind(&QuerySidePalette::get_help_topic_threaded, this, _1, 
     editor->current_statement(), caret));
@@ -509,7 +558,7 @@ bool QuerySidePalette::find_context_help(Sql_editor *editor)
  * to get the actual help text from it.
  */
 grt::StringRef QuerySidePalette::get_help_topic_threaded(grt::GRT *, const std::string &query,
-  std::pair<int, int> caret)
+  std::pair<ssize_t, ssize_t> caret)
 {
   SqlEditorForm::Ref form = _owner.lock();
   if (!form)
@@ -584,24 +633,7 @@ ToolBar* QuerySidePalette::prepare_snippet_toolbar()
   toolbar->set_padding(0, 0, 0, 0);
   toolbar->set_size(-1, 27);
 #endif
-  ToolBarItem* item = manage(new ToolBarItem(mforms::ActionItem));
-  item->set_name("add_snippet");
-  item->set_icon(App::get()->get_resource_path("snippet_add.png"));
-  item->set_tooltip(_("Add the current editor content as new snippet"));
-  scoped_connect(item->signal_activated(), boost::bind(&QuerySidePalette::snippet_toolbar_item_activated, this, _1));
-  toolbar->add_item(item);
-
-  /* not needed in toolbar
-  item = manage(new ToolBarItem(mforms::ActionItem));
-  item->set_name("del_snippet");
-  item->set_icon(App::get()->get_resource_path("snippet_del.png"));
-  item->set_tooltip(_("Delete the currently selected snippet"));
-  scoped_connect(item->signal_activated(), boost::bind(&QuerySidePalette::snippet_toolbar_item_activated, this, _1));
-  toolbar->add_item(item);
-   */
-
-  item = mforms::manage(new ToolBarItem(mforms::SeparatorItem));
-  toolbar->add_item(item);
+  ToolBarItem* item;
 
   item = mforms::manage(new ToolBarItem(mforms::SelectorItem));
   item->set_name("select_category");
@@ -662,7 +694,6 @@ void QuerySidePalette::snippet_toolbar_item_activated(ToolBarItem* item)
   }
 }
 
-
 //--------------------------------------------------------------------------------------------------
 
 void QuerySidePalette::snippet_selection_changed()
@@ -828,7 +859,7 @@ bool compare_lengths(const std::pair<std::string, std::string> &l, const std::pa
 
 //--------------------------------------------------------------------------------------------------
 
-void QuerySidePalette::check_format_structures(Sql_editor *editor)
+void QuerySidePalette::check_format_structures(MySQLEditor *editor)
 {
   mforms::CodeEditorConfig *config = editor->get_editor_settings();
 
@@ -1012,3 +1043,30 @@ std::string QuerySidePalette::format_help_as_html(const std::string &text)
 }
 
 //--------------------------------------------------------------------------------------------------
+
+void QuerySidePalette::refresh_snippets()
+{
+  if (_pending_snippets_refresh && _snippet_list->shared_snippets_active())
+  {
+    SqlEditorForm::Ref owner(_owner.lock());
+
+    try
+    {
+      _snippet_list->model()->load_from_db(owner.get());
+      _pending_snippets_refresh = false;
+    }
+    catch (std::exception &exc)
+    {
+      log_error("Error loading DB snippets: %s\n", exc.what());
+    }
+  }
+  _snippet_list->refresh_snippets();
+}
+
+
+//--------------------------------------------------------------------------------------------------
+
+void QuerySidePalette::edit_last_snippet()
+{
+  _snippet_list->edit_new_snippet();
+}
