@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "wb_live_schema_tree.h"
 #include "grtdb/charset_utils.h"
 #include "grt/icon_manager.h"
@@ -28,6 +26,8 @@
 #include "base/string_utilities.h"
 #include "base/log.h"
 
+#include "mforms/app.h"
+
 using namespace wb;
 using namespace bec;
 using namespace base;
@@ -932,7 +932,7 @@ bool LiveSchemaTree::update_node_children(mforms::TreeNodeRef parent, std::list<
                 // Start is reset to the iterator position
                 start = it;
 
-                last_position = position == -1 ? position : position + _node_collections[type].captions.size();
+                last_position = position == -1 ? position : position + (int)_node_collections[type].captions.size();
 
                 // Shortcut to quit once an item has been found to be located
                 // At the end of the list
@@ -1221,7 +1221,7 @@ void LiveSchemaTree::schema_contents_arrived(const std::string &schema_name,
           {
               for (size_t index = 0; index < (size_t)tables_node->count(); index++)
               {
-                  mforms::TreeNodeRef pnode = tables_node->get_child(index);
+                mforms::TreeNodeRef pnode = tables_node->get_child((int)index);
                   reload_object_data(pnode);
               }
           }
@@ -1232,7 +1232,7 @@ void LiveSchemaTree::schema_contents_arrived(const std::string &schema_name,
           {
               for (size_t index = 0; index < (size_t)views_node->count(); index++)
               {
-                  mforms::TreeNodeRef pnode = views_node->get_child(index);
+                mforms::TreeNodeRef pnode = views_node->get_child((int)index);
                   reload_object_data(pnode);
               }
           }
@@ -1515,6 +1515,7 @@ grt::BaseListRef LiveSchemaTree::get_selected_objects()
       switch (current_type)
       {
         case Any:
+        case None:
           break;
         case Schema:
           obj->type("db.Schema");
@@ -1685,11 +1686,11 @@ bec::MenuItemList LiveSchemaTree::get_popup_items_for_nodes(const std::list<mfor
         {
           DictRef options= DictRef::cast_from(_grt->get("/wb/options/options"));
           bool limit_rows= (0 != options.get_int("SqlEditor:LimitRows"));
-          int limit_rows_count= options.get_int("SqlEditor:LimitRowsCount");
+          ssize_t limit_rows_count= options.get_int("SqlEditor:LimitRowsCount");
           if (limit_rows && (limit_rows_count <= 0))
             limit_rows= false;
           if (limit_rows)
-            caption += strfmt(_(" - Limit %i"), limit_rows_count);
+            caption += _(" - Limit ") + base::to_string(limit_rows_count);
         }
         view_item.caption= caption;
       }
@@ -2067,6 +2068,8 @@ void LiveSchemaTree::set_model_view(mforms::TreeNodeView* target)
   {
     scoped_connect(_model_view->signal_expand_toggle(),boost::bind(&LiveSchemaTree::expand_toggled, this, _1, _2));
     scoped_connect(_model_view->signal_node_activated(),boost::bind(&LiveSchemaTree::node_activated, this, _1, _2));
+
+    _model_view->set_row_overlay_handler(boost::bind(&LiveSchemaTree::overlay_icons_for_tree_node, this, _1));
   }
 }
 
@@ -2270,6 +2273,8 @@ void LiveSchemaTree::expand_toggled(mforms::TreeNodeRef node, bool value)
   }
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void LiveSchemaTree::node_activated(mforms::TreeNodeRef node, int column)
 {
   LSTData *node_data = dynamic_cast<LSTData*> (node->get_data());
@@ -2278,30 +2283,114 @@ void LiveSchemaTree::node_activated(mforms::TreeNodeRef node, int column)
   {
     std::string node_name = node->get_string(0);
 
-    switch(node_data->get_type())
+    switch (node_data->get_type())
     {
     case Schema:
       {
         std::vector<ChangeRecord> changes;
-      
+
         ChangeRecord record = { Schema, "", node_name, "" };
         changes.push_back(record);
 
         if (boost::shared_ptr<Delegate> delegate = _delegate.lock())
-          delegate->tree_activate_objects("activate", changes);
+        {
+          switch (column)
+          {
+          case -1:
+            delegate->tree_activate_objects("inspect", changes);
+            break;
+          case -2:
+            delegate->tree_activate_objects("alter", changes);
+            break;
+          default:
+            delegate->tree_activate_objects("activate", changes);
+#ifndef _WIN32
+            node->toggle();
+#endif
+            break;
+          }
+        }
       }
       break;
+
+    case Table:
+    case View:
+      {
+        if (column < 0)
+        {
+          std::vector<ChangeRecord> changes;
+          ChangeRecord record = { node_data->get_type(), get_schema_name(node), node_name, "" };
+          changes.push_back(record);
+
+          if (boost::shared_ptr<Delegate> delegate = _delegate.lock())
+          {
+            switch (column)
+            {
+            case -1:
+              delegate->tree_activate_objects("inspect", changes);
+              break;
+            case -2:
+              delegate->tree_activate_objects("alter", changes);
+              break;
+            case -3:
+              delegate->tree_activate_objects("select_data", changes);
+              break;
+#ifndef _WIN32
+            default:
+              node->toggle();
+              break;
+#endif
+            }
+          }
+          break;
+        }
+        // else fall through.
+      }
+
+    case Procedure:
+    case Function:
+      {
+        if (column < 0)
+        {
+          std::vector<ChangeRecord> changes;
+          ChangeRecord record = { node_data->get_type(), get_schema_name(node), node_name, "" };
+          changes.push_back(record);
+
+          if (boost::shared_ptr<Delegate> delegate = _delegate.lock())
+          {
+            switch (column)
+            {
+            case -1:
+              delegate->tree_activate_objects("alter", changes);
+              break;
+            case -2:
+              delegate->tree_activate_objects("execute", changes);
+              break;
+            }
+          }
+          break;
+        }
+        // else fall through.
+      }
+
     default:
       node_name = base::quote_identifier_if_needed(node_name, '`');
       sql_editor_text_insert_signal(node_name);
       break;
+
     }
   }
+#ifndef _WIN32
+  else
+    node->toggle();
+#endif
 }
 
-/*
-* get_schema_name: finds the parent schema for a specific node in the tree
-*/
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Finds the parent schema for a specific node in the tree.
+ */
 std::string LiveSchemaTree::get_schema_name(const mforms::TreeNodeRef& node)
 {
   std::string ret_val;
@@ -2497,6 +2586,8 @@ void LiveSchemaTree::reload_object_data(mforms::TreeNodeRef& node)
     }
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void LiveSchemaTree::discard_object_data(mforms::TreeNodeRef& node, int data_mask)
 {
     mforms::TreeNodeRef parent_node;
@@ -2530,3 +2621,38 @@ void LiveSchemaTree::discard_object_data(mforms::TreeNodeRef& node, int data_mas
         parent_node->remove_children();
     }
 }
+
+//--------------------------------------------------------------------------------------------------
+
+std::vector<std::string> LiveSchemaTree::overlay_icons_for_tree_node(mforms::TreeNodeRef node)
+{
+  LSTData *data = dynamic_cast<LSTData*>(node->get_data());
+
+  std::vector<std::string> icons;
+  if (data)
+  {
+    switch (data->get_type())
+    {
+      case Schema:
+        icons.push_back(mforms::App::get()->get_resource_path("wb_item_overlay_inspector.png"));
+        icons.push_back(mforms::App::get()->get_resource_path("wb_item_overlay_editor.png"));
+        break;
+      case Table:
+      case View:
+        icons.push_back(mforms::App::get()->get_resource_path("wb_item_overlay_inspector.png"));
+        icons.push_back(mforms::App::get()->get_resource_path("wb_item_overlay_editor.png"));
+        icons.push_back(mforms::App::get()->get_resource_path("wb_item_overlay_result.png"));
+        break;
+      case Procedure:
+      case Function:
+        icons.push_back(mforms::App::get()->get_resource_path("wb_item_overlay_editor.png"));
+        icons.push_back(mforms::App::get()->get_resource_path("wb_item_overlay_execute.png"));
+        break;
+      default:
+        break;
+    }
+  }
+  return icons;
+}
+
+//--------------------------------------------------------------------------------------------------
