@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "recordset_text_storage.h"
 #include "recordset_be.h"
 #include "base/string_utilities.h"
@@ -68,6 +66,7 @@ static void process_templates(const std::list<std::string> &files)
       info.extension = cf.get_value("extension");
       info.description = cf.get_value("description");
       info.pre_quote_strings = cf.get_bool("pre_quote_strings");
+      info.quote = cf.get_value("quote");
       info.include_column_types = cf.get_value("include_column_types");
       info.null_syntax = cf.get_value("null_syntax");
       info.row_separator = cf.get_value("row_separator");
@@ -163,7 +162,7 @@ ColumnId Recordset_text_storage::aux_column_count()
 }
 
 
-void Recordset_text_storage::do_apply_changes(const Recordset *recordset, sqlite::connection *data_swap_db)
+void Recordset_text_storage::do_apply_changes(const Recordset *recordset, sqlite::connection *data_swap_db, bool skip_commitmig )
 {
   throw std::runtime_error("Recordset_text_storage::apply_changes is not implemented");
 }
@@ -231,11 +230,13 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
   const Recordset::Column_names *column_names= recordset->column_names();
   const Recordset::Column_types &column_types= get_column_types(recordset);
   const Recordset::Column_types &real_column_types= get_real_column_types(recordset);
-  const Recordset::Column_quoting &column_quoting= get_column_quoting(recordset);
+  const Recordset::Column_flags &column_flags= get_column_flags(recordset);
   
   ColumnId visible_col_count= recordset->get_column_count();
   sqlide::QuoteVar qv;
   {
+    if (info.quote != "")
+      qv.quote = info.quote;
     qv.escape_string= std::ptr_fun(escape_sql_string_);
     // swap db (sqlite) stores unknown values as quoted strings
     qv.store_unknown_as_string= true;
@@ -273,7 +274,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
       if (out_col_type == "String")
       {
         TemplateDictionary* col_index_dict = dict->AddSectionDictionary("STRING_COLUMN");
-        col_index_dict->SetIntValue("STRING_COLUMN_INDEX", col+1);
+        col_index_dict->SetIntValue("STRING_COLUMN_INDEX", (long)col + 1);
       }
     }
   }
@@ -326,7 +327,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
               TemplateDictionary* field_dict;
               ColumnId partition_column= col - col_begin;
               bool is_null;
-              v = data_rs->get_variant(partition_column);
+              v = data_rs->get_variant((int)partition_column);
 
               is_null = sqlide::is_var_null(v); // for some reason, the apply_visitor stuff isnt handling NULL
 
@@ -347,7 +348,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
               if (is_null)
                 field_value = null_syntax;
               else if (strings_are_pre_quoted)
-                field_value= column_quoting[col] || sqlide::is_var_null(v)?
+                field_value= (column_flags[col] & Recordset::NeedsQuoteFlag) || sqlide::is_var_null(v)?
                      boost::apply_visitor(qv, column_types[col], v):
                      boost::apply_visitor(var_to_str, v);
               else
@@ -369,7 +370,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
             std::string result_text;
             tpl->Expand(&result_text, &row_dict_base);
             
-            fprintf(outf, "%s", result_text.c_str());
+            fwrite(result_text.data(), 1, result_text.size(), outf);
           }                  
         }
         while (next_row_exists);
@@ -380,8 +381,8 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
     {
       std::string result_text;
       post_tpl->Expand(&result_text, dict.get());
-      
-      fprintf(outf, "%s", result_text.c_str());
+
+      fwrite(result_text.data(), 1, result_text.size(), outf);
     }
 
     fclose(outf);
@@ -408,14 +409,14 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
                  col_end= std::min<ColumnId>(visible_col_count, (partition + 1) * Recordset::DATA_SWAP_DB_TABLE_MAX_COL_COUNT); col < col_end; ++col)
             {
               ColumnId partition_column= col - col_begin;
-              v = data_rs->get_variant(partition_column);
+              v = data_rs->get_variant((int)partition_column);
               TemplateDictionary* field_dict = row_dict->AddSectionDictionary("FIELD");
               field_dict->SetValueWithoutCopy("FIELD_NAME", (*column_names)[col]);
               std::string field_value;
               sqlide::VarToStr var_to_str;
               
               if (strings_are_pre_quoted)
-                field_value= column_quoting[col] || sqlide::is_var_null(v)?
+                field_value= (column_flags[col] & Recordset::NeedsQuoteFlag) || sqlide::is_var_null(v)?
                     boost::apply_visitor(qv, column_types[col], v):
                     boost::apply_visitor(var_to_str, v);
               else
@@ -437,7 +438,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
       GError *error = 0;
       
       // use g_file_set because it can handle utf8 filenames
-      if (!g_file_set_contents(_file_path.c_str(), result_text.data(), result_text.size(), &error))
+      if (!g_file_set_contents(_file_path.c_str(), result_text.data(), (gssize)result_text.size(), &error))
       {
         std::string message = error->message;
         g_free(error);
