@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -194,6 +194,9 @@ namespace MySQL.GUI.Workbench
       // Initialize forms stuff.
       MySQL.Forms.Manager formsManager = MySQL.Forms.Manager.get_instance(); // Creates the singleton.
 
+      // init extra mforms things that are delegated to the frontend, indirectly through RecordsetWrapper in wbpublic
+      MySQL.Grt.Db.RecordsetWrapper.init_mforms(MySQL.Grt.Db.RecordsetView.create);
+
       #region Runtime path check
 
       // Currently WB has trouble running from a path containing non-ASCII characters.
@@ -274,9 +277,8 @@ namespace MySQL.GUI.Workbench
         mainForm = new MainForm(wbContext);
 
         // Initialize the Workbench context
-        AppImpl formsApplication = new AppImpl(
-          formsManager,
-          new AppImpl.StringAppCommandAppViewStringDelegate(mainForm.ApplicationCommand),
+        ManagedApplication formsApplication = new ManagedApplication(
+          new AppCommandDelegate(mainForm.ApplicationCommand),
           mainForm.dockDelegate);
 
         callbacks = new WbFrontendCallbacks(
@@ -284,7 +286,7 @@ namespace MySQL.GUI.Workbench
           new WbFrontendCallbacks.VoidStrDelegate(mainForm.ShowStatusText),
           new WbFrontendCallbacks.BoolStrStrFloatDelegate(mainForm.ShowProgress),
           new WbFrontendCallbacks.BoolStrIntStrPtrDelegate(mainForm.RequestInput),
-          new WbFrontendCallbacks.CanvasViewStringStringIntPtrDelegate(mainForm.CreateNewDiagram),
+          new WbFrontendCallbacks.CanvasViewStringStringDelegate(mainForm.CreateNewDiagram),
           new WbFrontendCallbacks.VoidCanvasViewDelegate(mainForm.DestroyView),
           new WbFrontendCallbacks.VoidCanvasViewDelegate(mainForm.SwitchedView),
           new WbFrontendCallbacks.VoidCanvasViewDelegate(mainForm.ToolChanged),
@@ -645,7 +647,6 @@ namespace MySQL.GUI.Workbench
       foreach (string subfolder in Directory.GetDirectories(folder))
         RemoveCompiledPythonFiles(subfolder, failed);
     }
-
     #endregion
 
   }
